/* SPDX-License-Identifier: GPL-2.0-or-later */

#ifndef _PCR_IDS_H_
#define _PCR_IDS_H_

#define PID_NOT_SUPPORTED 0xff
#define PID_ITSS          0x00
#define PID_RTC           PID_NOT_SUPPORTED
#define PID_PSF3          PID_NOT_SUPPORTED
#define PID_GPIOCOM0      0x03
#define PID_GPIOCOM5      PID_NOT_SUPPORTED
#define PID_GPIOCOM4      PID_NOT_SUPPORTED
#define PID_GPIOCOM3      PID_NOT_SUPPORTED
#define PID_GPIOCOM2      PID_NOT_SUPPORTED
#define PID_GPIOCOM1      PID_NOT_SUPPORTED

#endif	/* _PCR_IDS_H_ */
