/* SPDX-License-Identifier: GPL-2.0-only OR MIT */

#ifndef SOC_MEDIATEK_MT8188_CPU_INPUT_GATING_H
#define SOC_MEDIATEK_MT8188_CPU_INPUT_GATING_H

void disable_cpu_input_gating(void);

#endif /* SOC_MEDIATEK_MT8188_CPU_INPUT_GATING_H */
