/* SPDX-License-Identifier: GPL-2.0-or-later */

#ifndef __MAINBOARD_GPIO_H__
#define __MAINBOARD_GPIO_H__

#include <baseboard/gpio.h>

#define WWAN_FCPO	GPP_F21
#define WWAN_RST	GPP_E16
#define WWAN_PERST	GPP_E0
#define T1_OFF_MS	16
#define T2_OFF_MS	2

#endif /* __MAINBOARD_GPIO_H__ */
