/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef _SOC_DPTF_H_
#define _SOC_DPTF_H_

/* Below are the unique ACPI Device IDs for thermal/dptf on Apollo Lake SoC. */
/* DPTF ACPI Device ID */
#define DPTF_DPTF_DEVICE	"INT3400"
/* Generic ACPI Device ID for TSR0/1/2/3 and charger */
#define DPTF_GEN_DEVICE		"INT3403"
/* Fan ACPI Device ID */
#define DPTF_FAN_DEVICE		"INT3404"

#endif /* _SOC_DPTF_H_ */
