/*
 * Automatically generated C config: don't edit
 * libpayload version: 0.2.0
 * Sat Mar  9 10:22:45 2013
 */
#define AUTOCONF_INCLUDED
#define CONFIG_LP_SPEAKER 1
#define CONFIG_LP_STORAGE 1
#define CONFIG_LP_LIBC 1
#define CONFIG_LP_CBFS 1
#define CONFIG_LP_VGA_VIDEO_CONSOLE 1
#define CONFIG_LP_SERIAL_CONSOLE 1
#define CONFIG_LP_PC_KEYBOARD 1
#define CONFIG_LP_ARCH_X86 1
#define CONFIG_LP_STORAGE_ATA 1
#define CONFIG_LP_ARCH_SPECIFIC_OPTIONS 1
#define CONFIG_LP_STORAGE_AHCI_ONLY_TESTED 1
#define CONFIG_LP_8250_SERIAL_CONSOLE 1
#define CONFIG_LP_PDCURSES 1
#define CONFIG_LP_NVRAM 1
#define CONFIG_LP_PC_KEYBOARD_LAYOUT_US 1
#define CONFIG_LP_STORAGE_AHCI 1
#define CONFIG_LP_MULTIBOOT 1
#define CONFIG_LP_SERIAL_IOBASE 0x3f8
#define CONFIG_LP_CURSES 1
#define CONFIG_LP_CBMEM_CONSOLE 1
#define CONFIG_LP_LITTLE_ENDIAN 1
#define CONFIG_LP_PCI 1
#define CONFIG_LP_STORAGE_ATAPI 1
#define CONFIG_LP_VIDEO_CONSOLE 1
