/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef AMD_PICASSO_SOC_FSP_H
#define AMD_PICASSO_SOC_FSP_H

#include <fsp/api.h>

void mainboard_updm_update(FSP_M_CONFIG *mcfg);

#endif /* AMD_PICASSO_SOC_FSP_H */
