/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef __ARCH_MEMLAYOUT_H
#define __ARCH_MEMLAYOUT_H

/* Intel386 psABI requires a 16 byte aligned stack. */
#define ARCH_STACK_ALIGN_SIZE 16

#endif /* __ARCH_MEMLAYOUT_H */
