/* SPDX-License-Identifier: GPL-2.0-only OR MIT */

#ifndef SOC_MEDIATEK_MT8195_AUXADC_H
#define SOC_MEDIATEK_MT8195_AUXADC_H

#include <soc/auxadc_common.h>
#include <soc/infracfg.h>
#include <stdint.h>

static struct mt8195_infracfg_ao_regs *const mtk_infracfg = mt8195_infracfg_ao;

#endif
