/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef _SOC_XEON_SP_BOOTBLOCK_H_
#define _SOC_XEON_SP_BOOTBLOCK_H_

#include "iomap.h"

/* Bootblock post console init programming */
void bootblock_pch_init(void);
void report_platform_info(void);
#endif
