/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef __SOC_MEDIATEK_MT8195_INCLUDE_SOC_DPTX_H__
#define __SOC_MEDIATEK_MT8195_INCLUDE_SOC_DPTX_H__

#include <soc/dptx_common.h>

#define EDP_PHY_BASE	0

#endif /* __SOC_MEDIATEK_MT8195_INCLUDE_SOC_DPTX_H__ */
