/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef SOC_MEDIATEK_COMMON_CPU_ID_H
#define SOC_MEDIATEK_COMMON_CPU_ID_H

#define MTK_CPU_ID_MT8186G	0x81861001
#define MTK_CPU_ID_MT8186T	0x81862001

u32 get_cpu_id(void);

#endif /* SOC_MEDIATEK_COMMON_CPU_ID_H */
