/* SPDX-License-Identifier: GPL-2.0-only OR MIT */

#ifndef SOC_MEDIATEK_MT8188_WDT_H
#define SOC_MEDIATEK_MT8188_WDT_H

#include <soc/wdt_common.h>

#define MTK_WDT_SPM_THERMAL_VAL 1

DEFINE_BIT(MTK_WDT_SPM_THERMAL_EN, 0)
DEFINE_BIT(MTK_WDT_THERMAL_EN, 17)
DEFINE_BIT(MTK_WDT_THERMAL_IRQ, 17)

#endif /* SOC_MEDIATEK_MT8188_WDT_H */
