/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef GA_B75M_D3V_HDA_VERB_H
#define GA_B75M_D3V_HDA_VERB_H

const u32 cim_verb_data[] = {
	/* coreboot specific header */
	0x10ec0887,	// Realtek 887
	0x1458a002,	// Subsystem ID
	0x0000000e,	// Number of entries

	/* Pin Widget Verb Table */
	AZALIA_PIN_CFG(0, 0x11, AZALIA_PIN_CFG_NC(0)),
	AZALIA_PIN_CFG(0, 0x12, AZALIA_PIN_CFG_NC(0)),
	AZALIA_PIN_CFG(0, 0x14, 0x01014410),
	AZALIA_PIN_CFG(0, 0x15, AZALIA_PIN_CFG_NC(0)),
	AZALIA_PIN_CFG(0, 0x16, AZALIA_PIN_CFG_NC(0)),
	AZALIA_PIN_CFG(0, 0x17, AZALIA_PIN_CFG_NC(0)),
	AZALIA_PIN_CFG(0, 0x18, 0x01a19c50),
	AZALIA_PIN_CFG(0, 0x19, 0x02a19c60),
	AZALIA_PIN_CFG(0, 0x1a, 0x0181345f),
	AZALIA_PIN_CFG(0, 0x1b, 0x02214c20),
	AZALIA_PIN_CFG(0, 0x1c, AZALIA_PIN_CFG_NC(0)),
	AZALIA_PIN_CFG(0, 0x1d, 0x4004c601),
	AZALIA_PIN_CFG(0, 0x1e, AZALIA_PIN_CFG_NC(0)),
	AZALIA_PIN_CFG(0, 0x1f, AZALIA_PIN_CFG_NC(0))
};

#endif
