/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef SOUTHBRIDGE_INTEL_COMMON_HPET_H
#define SOUTHBRIDGE_INTEL_COMMON_HPET_H

#include <types.h>

void set_global_reset(const bool enable);

#endif /* SOUTHBRIDGE_INTEL_COMMON_HPET_H */
