/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef _SOC_SNOWRIDGE_BOOTBLOCK_H_
#define _SOC_SNOWRIDGE_BOOTBLOCK_H_

void early_uart_init(void);

void early_tco_init(void);

#endif // _SOC_SNOWRIDGE_BOOTBLOCK_H_
