/* SPDX-License-Identifier: CC-PDDC */

/* Please update the license if adding licensable material. */

/*
 * This is a placeholder, if there ever is something platform specific to
 * add to romstage, do it here.
 */
