/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef SOC_INTEL_SKL_ITSS_H
#define SOC_INTEL_SKL_ITSS_H

#include <intelblocks/itss.h>

#define GPIO_IRQ_START	50
#define GPIO_IRQ_END	ITSS_MAX_IRQ

#endif	/* SOC_INTEL_SKL_ITSS_H */
