/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef __SOC_NVIDIA_TEGRA210_MMU_OPERATIONS_H__
#define __SOC_NVIDIA_TEGRA210_MMU_OPERATIONS_H__

void tegra210_mmu_init(void);

#endif //__SOC_NVIDIA_TEGRA210_MMU_OPERATIONS_H__
