/* SPDX-License-Identifier: BSD-3-Clause */

#ifndef _COMMONLIB_BSD_GCD_H_
#define _COMMONLIB_BSD_GCD_H_

#include <stdint.h>

uint64_t gcd(uint64_t a, uint64_t b);

#endif /* _COMMONLIB_BSD_GCD_H_ */
