/* SPDX-License-Identifier: GPL-2.0-or-later */

#ifndef __VARIANT_EC_H__
#define __VARIANT_EC_H__

#include <baseboard/ec.h>

#undef MAINBOARD_EC_S3_WAKE_EVENTS
#define MAINBOARD_EC_S3_WAKE_EVENTS \
	(MAINBOARD_EC_S5_WAKE_EVENTS |\
	 EC_HOST_EVENT_MASK(EC_HOST_EVENT_AC_CONNECTED)    |\
	 EC_HOST_EVENT_MASK(EC_HOST_EVENT_AC_DISCONNECTED) |\
	 EC_HOST_EVENT_MASK(EC_HOST_EVENT_KEY_PRESSED)     |\
	 EC_HOST_EVENT_MASK(EC_HOST_EVENT_MKBP)            |\
	 EC_HOST_EVENT_MASK(EC_HOST_EVENT_MODE_CHANGE))

#undef MAINBOARD_EC_S0IX_WAKE_EVENTS
#define MAINBOARD_EC_S0IX_WAKE_EVENTS \
	(MAINBOARD_EC_S3_WAKE_EVENTS                       |\
	 EC_HOST_EVENT_MASK(EC_HOST_EVENT_USB_MUX)         |\
	 EC_HOST_EVENT_MASK(EC_HOST_EVENT_HANG_DETECT))

/* Enable EC backed Keyboard Backlight in ACPI */
#define EC_ENABLE_KEYBOARD_BACKLIGHT

#endif
