/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef __I2C_GENERIC_ADAU7002_CHIP_H__
#define __I2C_GENERIC_ADAU7002_CHIP_H__

struct drivers_generic_adau7002_config {
	/* Delay */
	unsigned int wakeup_delay;
};

#endif /* __I2C_GENERIC_ADAU7002_CHIP_H__ */
