/* SPDX-License-Identifier: GPL-2.0-only OR MIT */

/*
 * This file is created based on MT8189 Functional Specification
 * Chapter number: 9.17
 */

#ifndef __SOC_MEDIATEK_MT8189_INCLUDE_SOC_SPI_H__
#define __SOC_MEDIATEK_MT8189_INCLUDE_SOC_SPI_H__

#include <spi-generic.h>

#endif /* __SOC_MEDIATEK_MT8189_INCLUDE_SOC_SPI_H__ */
