/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef __MAINBOARD_SKU_H__
#define __MAINBOARD_SKU_H__

#define SKU_UNKNOWN	0xFFFFFFFF
#define SKU_0_NAMI	0x3A7B
#define SKU_0_VAYNE	0x3A67
#define SKU_1_VAYNE	0x3A63
#define SKU_2_VAYNE	0x3A7F
#define SKU_0_PANTHEON	0x3F63
#define SKU_1_PANTHEON	0x3F67
#define SKU_2_PANTHEON	0x3F62
#define SKU_3_PANTHEON  0x100BF63
#define SKU_4_PANTHEON  0x100BF62
#define SKU_0_SONA	0x3AE2
#define SKU_1_SONA	0x3AE3
#define SKU_0_AKALI	0x29E1
#define SKU_1_AKALI	0x2861
#define SKU_0_AKALI360	0x2BE7
#define SKU_1_AKALI360	0x2A67
#define SKU_0_SYNDRA	0x2BC63
#define SKU_1_SYNDRA	0x2BC62
#define SKU_2_SYNDRA	0x2BC61
#define SKU_3_SYNDRA	0x2BC60
#define SKU_4_SYNDRA    0x6BC63
#define SKU_5_SYNDRA    0x6BC62
#define SKU_6_SYNDRA    0x6BC61
#define SKU_7_SYNDRA    0x6BC60
#define SKU_0_EKKO	0x10118E3
#define SKU_1_EKKO	0x10018E3
#define SKU_2_EKKO	0x10118E1
#define SKU_3_EKKO	0x10018E1
#define SKU_4_EKKO	0x10118E2
#define SKU_5_EKKO	0x10018E2
#define SKU_6_EKKO	0x10118E0
#define SKU_7_EKKO	0x10018E0
#define SKU_0_BARD	0x1019CE3
#define SKU_1_BARD	0x1009CE3
#define SKU_2_BARD	0x1019CE1
#define SKU_3_BARD	0x1009CE1
#define SKU_4_BARD	0x1009CE0
#define SKU_5_BARD	0x1009CE2
#define SKU_6_BARD	0x1019CE0
#define SKU_7_BARD	0x1019CE2

#endif /* __MAINBOARD_SKU_H__ */
