/* SPDX-License-Identifier: GPL-2.0-only */

#if CONFIG(VBOOT_STARTS_BEFORE_BOOTBLOCK)
	ALIGN_COUNTER(64)
	_transfer_buffer = .;
	REGION(transfer_info, ., TRANSFER_INFO_SIZE, 4)
#endif

#if CONFIG(VBOOT_STARTS_BEFORE_BOOTBLOCK) || CONFIG(VBOOT_STARTS_IN_BOOTBLOCK)
	ALIGN_COUNTER(64)
	VBOOT2_WORK(., VB2_FIRMWARE_WORKBUF_RECOMMENDED_SIZE)
#endif

	ALIGN_COUNTER(64)
#if CONFIG(VBOOT_STARTS_BEFORE_BOOTBLOCK)
#if ENV_SEPARATE_VERSTAGE
	PRERAM_CBMEM_CONSOLE(., CONFIG_PRE_X86_CBMEM_CONSOLE_SIZE)
#else
	REGION(cbmemc_transfer, ., CONFIG_PRE_X86_CBMEM_CONSOLE_SIZE, 4)
#endif /* ENV_SEPARATE_VERSTAGE */
#endif /* CONFIG(VBOOT_STARTS_BEFORE_BOOTBLOCK) */
	TIMESTAMP(., TIMESTAMP_BUFFER_SIZE)
	CBFS_MCACHE(., CONFIG_CBFS_MCACHE_SIZE)
	FMAP_CACHE(., FMAP_SIZE)

#if CONFIG(VBOOT_STARTS_BEFORE_BOOTBLOCK)
	_etransfer_buffer = .;
#endif
