/* SPDX-License-Identifier: GPL-2.0-or-later */

#ifndef VARIANT_GPIO_H
#define VARIANT_GPIO_H

#include <baseboard/gpio.h>

#undef GPIO_EC_IN_RW
/* EC in RW */
#define GPIO_EC_IN_RW		GPP_F17

#endif
