/* SPDX-License-Identifier: GPL-2.0-or-later */

#ifndef __VARIANT_EC_H__
#define __VARIANT_EC_H__

#include <baseboard/ec.h>

/* Enable EC backed Keyboard Backlight in ACPI */
#define EC_ENABLE_KEYBOARD_BACKLIGHT

/* Enable DPTF OEM variable */
#define DPTF_ENABLE_OEM_VARIABLES

#endif
