/* SPDX-License-Identifier: GPL-2.0-only */

/* Enable EC backed ALS device in ACPI */
#define EC_ENABLE_ALS_DEVICE

/* Enable EC backed Keyboard Backlight in ACPI */
#define EC_ENABLE_KEYBOARD_BACKLIGHT

/* EC ENABLE MULTIPLE DPTF PROFILES */
#define EC_ENABLE_MULTIPLE_DPTF_PROFILES

/* Enable Tablet switch */
#define EC_ENABLE_TBMC_DEVICE
