/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef SOC_INTEL_COMMON_BLOCK_SMBUS_H
#define SOC_INTEL_COMMON_BLOCK_SMBUS_H

/* Program SMBus IO base, enable host Controller interface, clear status reg */
void smbus_common_init(void);

#endif	/* SOC_INTEL_COMMON_BLOCK_SMBUS_H */
