/* SPDX-License-Identifier: GPL-2.0-only */

/* SKU ID enumeration */
enum careena_sku {
	SKU_UNKNOWN = -1,
	SKU_CAREENA_KB_NO_BACKLIGHT16 = 16,
	SKU_CAREENA_KB_BACKLIGHT18 = 18,
	SKU_CAREENA_KB_BACKLIGHT19 = 19,
	SKU_CAREENA_KB_BACKLIGHT22 = 22,
	SKU_CAREENA_KB_BACKLIGHT23 = 23,
	SKU_BARLA_ALC5682_44 = 44,
	SKU_BARLA_ALC5682_45 = 45,
	SKU_BARLA_ALC5682_46 = 46,
	SKU_BARLA_ALC5682_47 = 47,
};
