/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef _SOC_EHL_IRQ_H_
#define _SOC_EHL_IRQ_H_

#define GPIO_IRQ14 14
#define GPIO_IRQ15 15

#define PCH_IRQ10 10
#define PCH_IRQ11 11
#define PCH_IRQ24 24
#define PCH_IRQ25 25
#define PCH_IRQ26 26
#define PCH_IRQ27 27
#define PCH_IRQ28 28
#define PCH_IRQ29 29
#define PCH_IRQ30 30
#define PCH_IRQ31 31
#define PCH_IRQ32 32
#define PCH_IRQ33 33
#define PCH_IRQ34 34
#define PCH_IRQ35 35
#define PCH_IRQ36 36

/* PCI shared IRQs */
#define PCH_IRQ_A 16
#define PCH_IRQ_B 17
#define PCH_IRQ_C 18
#define PCH_IRQ_D 19
#define PCH_IRQ_E 20
#define PCH_IRQ_F 21
#define PCH_IRQ_G 22
#define PCH_IRQ_H 23

/* LPSS Device IRQs */
#define LPSS_UART0_IRQ	16
#define LPSS_UART1_IRQ	17
#define LPSS_UART2_IRQ	33

#endif /* _EHL_IRQ_H_ */
