/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef __MAINBOARD_EC_H__
#define __MAINBOARD_EC_H__

#include <baseboard/ec.h>

#define EC_ENABLE_MULTIPLE_DPTF_PROFILES /* Config for multi-DPTF profiles */

#endif /* __MAINBOARD_EC_H__ */
