/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef DRIVERS_GENESYSLOGIC_GL9750_H
#define DRIVERS_GENESYSLOGIC_GL9750_H

/* Definitions for Genesys Logic GL9750 */

#define CFG		0x800
#define   CFG_EN	0x1
#define CFG1			0x844
#define   CFG1_WP_INVERT	(0x1 << 31)
#define CFG2			0x848
#define   CFG2_L0S_SUPPORT	(0x1 << 6)

#endif /* DRIVERS_GENESYSLOGIC_GL9750_H */
