/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef _SOC_SNOWRIDGE_PCR_IDS_H_
#define _SOC_SNOWRIDGE_PCR_IDS_H_

#define PID_PCIE2_P0 0x71
#define PID_PSTH     0x89
#define PID_ESPISPI  0x93
#define PID_USB2     0xA7
#define PID_PCIE0_P0 0xB4
#define PID_MODPHY1  0xBA
#define PID_MODPHY2  0xBB
#define PID_GPIOCOM1 0xC2 /**< GPIO Community West. */
#define PID_GPIOCOM0 0xC5 /**< GPIO Community East. */
#define PID_CC       0xCC
#define PID_SMB      0xCF
#define PID_ITSS     0xD0
#define PID_LPC      0xD2
#define PID_RTC      0xD1
#define PID_DMI      0x4D

#endif // _SOC_SNOWRIDGE_PCR_IDS_H_
