/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef __MAINBOARD_SKU_H__
#define __MAINBOARD_SKU_H__

#define SKU_UNKNOWN		0xFFFFFFFF
#define SKU_0_NAUTILUS		0x0000
#define SKU_1_NAUTILUS_LTE	0x0001

#endif /* __MAINBOARD_SKU_H__ */
