/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef HARCUVAR_MAINBOARD_BOARD_H
#define HARCUVAR_MAINBOARD_BOARD_H

#include <stdint.h>

#define BoardIdHarcuvar 0x52

uint32_t board_id(void);

#endif /* MAINBOARD_BOARD_H */
