/* SPDX-License-Identifier: GPL-2.0-only */

#include <device/pci_type.h>

#ifndef SOC_INTEL_COMMON_BLOCK_IMC_H
#define SOC_INTEL_COMMON_BLOCK_IMC_H

void imc_spd_smbus_init(pci_devfn_t dev);

#endif
