/* SPDX-License-Identifier: GPL-2.0-or-later */

#ifndef VARIANT_GPIO_H
#define VARIANT_GPIO_H

#include <baseboard/gpio.h>
#define WWAN_FCPO	GPP_F21
#define WWAN_RST	GPP_E16
#define WWAN_PERST	GPP_E0
#define T1_OFF_MS	16
#define T2_OFF_MS	2

#endif
