/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef __SOC_ELKHARTLAKE_PCIE_H__
#define __SOC_ELKHARTLAKE_PCIE_H__

#include <intelblocks/pcie_rp.h>

extern const struct pcie_rp_group pch_rp_groups[];

#endif /* __SOC_ELKHARTLAKE_PCIE_H__ */
