/* SPDX-License-Identifier: GPL-2.0-or-later */

#ifndef AMD_BLOCK_GRAPHICS_H
#define AMD_BLOCK_GRAPHICS_H

#include <device/device.h>

void fsp_graphics_init(struct device *const dev);

#endif /* AMD_BLOCK_GRAPHICS_H */
