/* SPDX-License-Identifier: GPL-2.0-or-later */

#ifndef VARIANT_GPIO_H
#define VARIANT_GPIO_H

#include <baseboard/gpio.h>

#define WWAN_FCPO	GPP_D6
#define WWAN_RST	GPP_F12
#define T2_OFF_MS	20

#endif
