/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef SECURITY_INTEL_TXT_LIB_H_
#define SECURITY_INTEL_TXT_LIB_H_

#include <types.h>

bool is_establishment_bit_asserted(void);

bool is_txt_cpu(void);

#endif /* SECURITY_INTEL_TXT_LIB_H_ */
