/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef SOC_INTEL_COMMON_BLOCK_CAR_LIB_H
#define SOC_INTEL_COMMON_BLOCK_CAR_LIB_H

#include <types.h>

/* Gathers and prints information about the CPU's L3 cache */
void car_report_cache_info(void);

#endif	/* SOC_INTEL_COMMON_BLOCK_CAR_LIB_H */
