/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef _SOC_LPM_H_
#define _SOC_LPM_H_

#include <soc/soc_chip.h>

int get_supported_lpm_mask(struct soc_intel_tigerlake_config *config);

#endif /* _SOC_LPM_H_ */
