/* SPDX-License-Identifier: GPL-2.0-or-later */

#ifndef _PCR_IDS_H_
#define _PCR_IDS_H_

#define PID_CSME0	0x90
#define PID_ITSS	0xC4
#define PID_RTC		0xC3
#define PID_PSF3	0xBC
#define PID_DMI		0xEF
#define PID_GPIOCOM5	0x11
#define PID_GPIOCOM4	0xAB
#define PID_GPIOCOM3	0xAC
#define PID_GPIOCOM2	0xAD
#define PID_GPIOCOM1	0xAE
#define PID_GPIOCOM0	0xAF

#endif	/* _PCR_IDS_H_ */
