/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef SOC_MEDIATEK_TRACKER_V1_H
#define SOC_MEDIATEK_TRACKER_V1_H

#define SYS_TRACK_ENTRY			8

#define AR_TRACK_OFFSET			0x0100
#define AW_TRACK_OFFSET			0x0200

enum {
	TRACKER_SYSTRACKER = 0,
	TRACKER_NUM,
};

#endif
