/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef ATLAS_MAINBOARD_H
#define ATLAS_MAINBOARD_H

/* Embedded controller settings */
#define EC_IDX_PORT		0x2e
#define EMI_0_IO_BASE_ADDR	0xc00

#endif /* ATLAS_MAINBOARD_H */
