/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef AMD_CEZANNE_UART_H
#define AMD_CEZANNE_UART_H

#include <types.h>

void clear_uart_legacy_config(void); /* disable legacy I/O decode for FCH UART */

#endif /* AMD_CEZANNE_UART_H */
