/* SPDX-License-Identifier: GPL-2.0-only */

#include <soc/platform_descriptors.h>

void mainboard_get_dxio_ddi_descriptors(
		const fsp_dxio_descriptor **dxio_descs, size_t *dxio_num,
		const fsp_ddi_descriptor **ddi_descs, size_t *ddi_num)
{
}
