/* SPDX-License-Identifier: GPL-2.0-only */

#include <amdblocks/agesawrapper.h>
#include <amdblocks/BiosCallOuts.h>
#include <soc/southbridge.h>

void platform_FchParams_env(FCH_DATA_BLOCK *FchParams_env)
{
}
