/* Copyright 2019 The ChromiumOS Authors
 * Use of this source code is governed by a BSD-style license that can be
 * found in the LICENSE file.
 *
 * This header may be imported to expose a limited set of constants to
 * assembly files or linker scripts without including the entire vboot API.
 */

#ifndef VBOOT_REFERENCE_VB2_CONSTANTS_H_
#define VBOOT_REFERENCE_VB2_CONSTANTS_H_

#include "../2lib/include/2constants.h"

#endif  /* VBOOT_REFERENCE_VB2_CONSTANTS_H_ */
