/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef _DENVERTON_NS_SMM_H_
#define _DENVERTON_NS_SMM_H_

struct smm_relocation_attrs {
	uint32_t smbase;
	uint32_t smrr_base;
	uint32_t smrr_mask;
};

#endif /* _DENVERTON_NS_SMM_H_ */
