/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef WS_2_ONBOARD_H
#define WS_2_ONBOARD_H

#define E810_CAM_NUMBER		2
#define E810_CAM1_PORT_NUMBER	4
#define E810_CAM2_PORT_NUMBER	8
#define E810_CAM_PORT_NUMBER	(E810_CAM1_PORT_NUMBER + E810_CAM2_PORT_NUMBER)
#define E810_CAM_PORT_NAME_LENGTH	15
#define E810_CAM1_BUS_NUMBER	0x43
#define E810_CAM2_BUS_NUMBER	0x44

#endif /* End of WS_2_ONBOARD_H */
