/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef MAINBOARD_GPIO_H
#define MAINBOARD_GPIO_H

#include <baseboard/gpio.h>

/* EC sync irq is GPP_134_IRQ */
#define EC_SYNC_IRQ	GPIO_134_IRQ

#endif /* MAINBOARD_GPIO_H */
