/* SPDX-License-Identifier: GPL-2.0-only */

/*
 * MIPI Alliance Manufacturer IDs from https://mid.mipi.org
 */

#ifndef __MIPI_IDS_H__
#define __MIPI_IDS_H__

/* Board Members */
#define MIPI_MFG_ID_INTEL			0x0105
#define MIPI_MFG_ID_QUALCOMM			0x0217
#define MIPI_MFG_ID_BOSCH			0x03b8
#define MIPI_MFG_ID_SAMSUNG			0x010b
#define MIPI_MFG_ID_ST_MICRO			0x0104
#define MIPI_MFG_ID_SYNOPSYS			0x0148
#define MIPI_MFG_ID_TI				0x0102
#define MIPI_MFG_ID_TOSHIBA			0x0126

/* Contributing Members */
#define MIPI_MFG_ID_REALTEK			0x025d
#define MIPI_DEV_ID_REALTEK_ALC5682		0x5682
#define MIPI_DEV_ID_REALTEK_ALC711		0x0711
#define MIPI_DEV_ID_REALTEK_ALC721		0x0721
#define MIPI_DEV_ID_REALTEK_ALC722		0x0722
#define MIPI_DEV_ID_REALTEK_ALC1308             0x1308

#define MIPI_MFG_ID_MAXIM			0x019f
#define MIPI_DEV_ID_MAXIM_MAX98373		0x8373
#define MIPI_DEV_ID_MAXIM_MAX98363		0x8363

#define MIPI_MFG_ID_CIRRUS			0x01fa
#define MIPI_DEV_ID_CIRRUS_CS42L42		0x4242

#endif /* __MIPI_IDS_H__ */
