/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef SOC_SAMSUNG_COMMON_INCLUDE_SOC_I2C_H
#define SOC_SAMSUNG_COMMON_INCLUDE_SOC_I2C_H

void i2c_init(unsigned int bus, int speed, int slaveadd);

#endif /* SOC_SAMSUNG_COMMON_INCLUDE_SOC_I2C_H */
