/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef _SOC_PANTHERLAKE_IRQ_H_
#define _SOC_PANTHERLAKE_IRQ_H_

#define GPIO_IRQ14	14
#define GPIO_IRQ15	15

#define PCH_IRQ10	10
#define PCH_IRQ11	11

#endif
