/* SPDX-License-Identifier: GPL-2.0-only OR MIT */

#ifndef __SOC_MEDIATEK_MT8189_INCLUDE_SOC_EMI_H__
#define __SOC_MEDIATEK_MT8189_INCLUDE_SOC_EMI_H__

#include <stddef.h>

size_t sdram_size(void);

#endif /* __SOC_MEDIATEK_MT8189_INCLUDE_SOC_EMI_H__ */
