/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef THINKPAD_T410_DOCK_H
#define THINKPAD_T410_DOCK_H
void init_dock(void);
void dock_connect(void);
void dock_disconnect(void);
int dock_present(void);
#endif
