/* SPDX-License-Identifier: GPL-2.0-only */

/* I2C */
#define I2C_0_SPEED 100000
#define I2C_SLAVE 0

/* Voltages */
#define VDD_ARM_MV 1300		// 1.3V
#define VDD_INT_UV 1012500	// 1.0125V
#define VDD_MIF_MV 1000		// 1.0V
#define VDD_G3D_MV 1200		// 1.2V
#define VDD_LDO2_MV 1500	// 1.5V
#define VDD_LDO3_MV 1800	// 1.8V
#define VDD_LDO5_MV 1800	// 1.8V
#define VDD_LDO10_MV 1800	// 1.8V
