/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef _SOC_SNOWRIDGE_KTI_CACHE_H_
#define _SOC_SNOWRIDGE_KTI_CACHE_H_

#include <stddef.h>

void *kti_cache_load(size_t *size);

#endif // _SOC_SNOWRIDGE_KTI_CACHE_H_
