/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef SOC_ELKHARTLAKE_PCR_H
#define SOC_ELKHARTLAKE_PCR_H
/*
 * Port IDs
 */
#define PID_EMMC	0x51
#define PID_SDX		0x53

#define PID_GPIOCOM0	0x6e
#define PID_GPIOCOM1	0x6d
#define PID_GPIOCOM2	0x6c
#define PID_GPIOCOM3	0x6b
#define PID_GPIOCOM4	0x6a
#define PID_GPIOCOM5	0x69

#define PID_DMI		0x88
#define PID_PSTH	0x89
#define PID_CSME0	0x90
#define PID_ISCLK	0xad
#define PID_PSF1	0xba
#define PID_PSF2	0xbb
#define PID_PSF3	0xbc
#define PID_PSF4	0xbd
#define PID_RTC		0xc3
#define PID_ITSS	0xc4
#define PID_ESPI	0x72
#define PID_SERIALIO	0xcb

#endif
