/* SPDX-License-Identifier: GPL-2.0-only OR MIT */

/*
 * This file is created based on MT8188 Functional Specification
 * Chapter number: 5.13
 */

#ifndef SOC_MEDIATEK_MT8188_TIMER_H
#define SOC_MEDIATEK_MT8188_TIMER_H

#include <soc/timer_v2.h>

#endif
