/* SPDX-License-Identifier: GPL-2.0-only OR MIT */

#ifndef SOC_MEDIATEK_MT8196_THERMAL_H
#define SOC_MEDIATEK_MT8196_THERMAL_H

void thermal_sram_init(void);
void thermal_init(void);

#endif /* SOC_MEDIATEK_MT8196_THERMAL_H */
