/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef _SOC_P2SB_H_
#define _SOC_P2SB_H_

/* PCI config space registers */
#define HPTC_OFFSET		0x60
#define HPTC_ADDR_ENABLE_BIT	(1 << 7)

#define PCH_P2SB_EPMASK0	0xB0

#endif /* _SOC_P2SB_H_ */
