/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef __SOC_MEDIATEK_MT8188_INCLUDE_SOC_DPTX_REG_H__
#define __SOC_MEDIATEK_MT8188_INCLUDE_SOC_DPTX_REG_H__

#include <soc/dptx_reg_common.h>

#endif /* __SOC_MEDIATEK_MT8188_INCLUDE_SOC_DPTX_REG_H__ */
