/* SPDX-License-Identifier: Apache-2.0 */
/* SPDX-License-Identifier: GPL-2.0-or-later */
/* See the file LICENSE for further information */

#define               DENALI_CTL_00_DATA 0x00000a00
#define               DENALI_CTL_01_DATA 0x00000000
#define               DENALI_CTL_02_DATA 0x00000000
#define               DENALI_CTL_03_DATA 0x00000000
#define               DENALI_CTL_04_DATA 0x00000000
#define               DENALI_CTL_05_DATA 0x00000000
#define               DENALI_CTL_06_DATA 0x0000000a
#define               DENALI_CTL_07_DATA 0x0002d362
#define               DENALI_CTL_08_DATA 0x00071073
#define               DENALI_CTL_09_DATA 0x0a1c0255
#define               DENALI_CTL_10_DATA 0x1c1c0400
#define               DENALI_CTL_11_DATA 0x0404990b
#define               DENALI_CTL_12_DATA 0x2b050405
#define               DENALI_CTL_13_DATA 0x0e0c081e
#define               DENALI_CTL_14_DATA 0x08090914
#define               DENALI_CTL_15_DATA 0x00fde718
#define               DENALI_CTL_16_DATA 0x00180a05
#define               DENALI_CTL_17_DATA 0x008b130e
#define               DENALI_CTL_18_DATA 0x01000118
#define               DENALI_CTL_19_DATA 0x0e032101
#define               DENALI_CTL_20_DATA 0x00000000
#define               DENALI_CTL_21_DATA 0x00000101
#define               DENALI_CTL_22_DATA 0x00000000
#define               DENALI_CTL_23_DATA 0x0a000000
#define               DENALI_CTL_24_DATA 0x00000000
#define               DENALI_CTL_25_DATA 0x01450100
#define               DENALI_CTL_26_DATA 0x00001c36
#define               DENALI_CTL_27_DATA 0x00000005
#define               DENALI_CTL_28_DATA 0x00170006
#define               DENALI_CTL_29_DATA 0x014e0300
#define               DENALI_CTL_30_DATA 0x03010000
#define               DENALI_CTL_31_DATA 0x000a0e00
#define               DENALI_CTL_32_DATA 0x04030200
#define               DENALI_CTL_33_DATA 0x0000031f
#define               DENALI_CTL_34_DATA 0x00070004
#define               DENALI_CTL_35_DATA 0x00000000
#define               DENALI_CTL_36_DATA 0x00000000
#define               DENALI_CTL_37_DATA 0x00000000
#define               DENALI_CTL_38_DATA 0x00000000
#define               DENALI_CTL_39_DATA 0x00000000
#define               DENALI_CTL_40_DATA 0x00000000
#define               DENALI_CTL_41_DATA 0x00000000
#define               DENALI_CTL_42_DATA 0x00000000
#define               DENALI_CTL_43_DATA 0x00000000
#define               DENALI_CTL_44_DATA 0x00000000
#define               DENALI_CTL_45_DATA 0x00000000
#define               DENALI_CTL_46_DATA 0x00000000
#define               DENALI_CTL_47_DATA 0x00000000
#define               DENALI_CTL_48_DATA 0x00000000
#define               DENALI_CTL_49_DATA 0x00000000
#define               DENALI_CTL_50_DATA 0x00000000
#define               DENALI_CTL_51_DATA 0x00000000
#define               DENALI_CTL_52_DATA 0x00000000
#define               DENALI_CTL_53_DATA 0x00000000
#define               DENALI_CTL_54_DATA 0x00000000
#define               DENALI_CTL_55_DATA 0x00000000
#define               DENALI_CTL_56_DATA 0x00000000
#define               DENALI_CTL_57_DATA 0x00000000
#define               DENALI_CTL_58_DATA 0x00000000
#define               DENALI_CTL_59_DATA 0x00000000
#define               DENALI_CTL_60_DATA 0x00000424
#define               DENALI_CTL_61_DATA 0x00000201
#define               DENALI_CTL_62_DATA 0x00001008
#define               DENALI_CTL_63_DATA 0x00000000
#define               DENALI_CTL_64_DATA 0x00000200
#define               DENALI_CTL_65_DATA 0x00000000
#define               DENALI_CTL_66_DATA 0x00000481
#define               DENALI_CTL_67_DATA 0x00000400
#define               DENALI_CTL_68_DATA 0x00000424
#define               DENALI_CTL_69_DATA 0x00000201
#define               DENALI_CTL_70_DATA 0x00001008
#define               DENALI_CTL_71_DATA 0x00000000
#define               DENALI_CTL_72_DATA 0x00000200
#define               DENALI_CTL_73_DATA 0x00000000
#define               DENALI_CTL_74_DATA 0x00000481
#define               DENALI_CTL_75_DATA 0x00000400
#define               DENALI_CTL_76_DATA 0x01010000
#define               DENALI_CTL_77_DATA 0x00000000
#define               DENALI_CTL_78_DATA 0x00000000
#define               DENALI_CTL_79_DATA 0x00000000
#define               DENALI_CTL_80_DATA 0x00000000
#define               DENALI_CTL_81_DATA 0x00000000
#define               DENALI_CTL_82_DATA 0x00000000
#define               DENALI_CTL_83_DATA 0x00000000
#define               DENALI_CTL_84_DATA 0x00000000
#define               DENALI_CTL_85_DATA 0x00000000
#define               DENALI_CTL_86_DATA 0x00000000
#define               DENALI_CTL_87_DATA 0x00000000
#define               DENALI_CTL_88_DATA 0x00000000
#define               DENALI_CTL_89_DATA 0x00000000
#define               DENALI_CTL_90_DATA 0x00000000
#define               DENALI_CTL_91_DATA 0x00000000
#define               DENALI_CTL_92_DATA 0x00000000
#define               DENALI_CTL_93_DATA 0x00000000
#define               DENALI_CTL_94_DATA 0x00000000
#define               DENALI_CTL_95_DATA 0x00000000
#define               DENALI_CTL_96_DATA 0x00000000
#define               DENALI_CTL_97_DATA 0x00000000
#define               DENALI_CTL_98_DATA 0x00000000
#define               DENALI_CTL_99_DATA 0x00000000
#define              DENALI_CTL_100_DATA 0x00000000
#define              DENALI_CTL_101_DATA 0x00000000
#define              DENALI_CTL_102_DATA 0x00000000
#define              DENALI_CTL_103_DATA 0x00000000
#define              DENALI_CTL_104_DATA 0x00000000
#define              DENALI_CTL_105_DATA 0x00000003
#define              DENALI_CTL_106_DATA 0x00000000
#define              DENALI_CTL_107_DATA 0x00000000
#define              DENALI_CTL_108_DATA 0x00000000
#define              DENALI_CTL_109_DATA 0x00000000
#define              DENALI_CTL_110_DATA 0x01000000
#define              DENALI_CTL_111_DATA 0x00040000
#define              DENALI_CTL_112_DATA 0x00800200
#define              DENALI_CTL_113_DATA 0x00000200
#define              DENALI_CTL_114_DATA 0x00000040
#define              DENALI_CTL_115_DATA 0x01000100
#define              DENALI_CTL_116_DATA 0x0a000002
#define              DENALI_CTL_117_DATA 0x0101ffff
#define              DENALI_CTL_118_DATA 0x01010101
#define              DENALI_CTL_119_DATA 0x01010101
#define              DENALI_CTL_120_DATA 0x0000010b
#define              DENALI_CTL_121_DATA 0x00000c01
#define              DENALI_CTL_122_DATA 0x00000000
#define              DENALI_CTL_123_DATA 0x00000000
#define              DENALI_CTL_124_DATA 0x00000000
#define              DENALI_CTL_125_DATA 0x00000000
#define              DENALI_CTL_126_DATA 0x00030300
#define              DENALI_CTL_127_DATA 0x00000000
#define              DENALI_CTL_128_DATA 0x00010001
#define              DENALI_CTL_129_DATA 0x00000000
#define              DENALI_CTL_130_DATA 0x00000000
#define              DENALI_CTL_131_DATA 0x00000000
#define              DENALI_CTL_132_DATA 0x00000000
#define              DENALI_CTL_133_DATA 0x00000000
#define              DENALI_CTL_134_DATA 0x00000000
#define              DENALI_CTL_135_DATA 0x00000000
#define              DENALI_CTL_136_DATA 0x00000000
#define              DENALI_CTL_137_DATA 0x00000000
#define              DENALI_CTL_138_DATA 0x00000000
#define              DENALI_CTL_139_DATA 0x00000000
#define              DENALI_CTL_140_DATA 0x00000000
#define              DENALI_CTL_141_DATA 0x00000000
#define              DENALI_CTL_142_DATA 0x00000000
#define              DENALI_CTL_143_DATA 0x00000000
#define              DENALI_CTL_144_DATA 0x00000000
#define              DENALI_CTL_145_DATA 0x00000000
#define              DENALI_CTL_146_DATA 0x00000000
#define              DENALI_CTL_147_DATA 0x00000000
#define              DENALI_CTL_148_DATA 0x00000000
#define              DENALI_CTL_149_DATA 0x00000000
#define              DENALI_CTL_150_DATA 0x00000000
#define              DENALI_CTL_151_DATA 0x00000000
#define              DENALI_CTL_152_DATA 0x00000000
#define              DENALI_CTL_153_DATA 0x00000000
#define              DENALI_CTL_154_DATA 0x00000000
#define              DENALI_CTL_155_DATA 0x00000000
#define              DENALI_CTL_156_DATA 0x00000000
#define              DENALI_CTL_157_DATA 0x00000000
#define              DENALI_CTL_158_DATA 0x00000000
#define              DENALI_CTL_159_DATA 0x00000000
#define              DENALI_CTL_160_DATA 0x00000000
#define              DENALI_CTL_161_DATA 0x02010102
#define              DENALI_CTL_162_DATA 0x0107070d
#define              DENALI_CTL_163_DATA 0x04040400
#define              DENALI_CTL_164_DATA 0x03000503
#define              DENALI_CTL_165_DATA 0x00000000
#define              DENALI_CTL_166_DATA 0x00000000
#define              DENALI_CTL_167_DATA 0x00000000
#define              DENALI_CTL_168_DATA 0x00000000
#define              DENALI_CTL_169_DATA 0x280d0000
#define              DENALI_CTL_170_DATA 0x01000000
#define              DENALI_CTL_171_DATA 0x00000000
#define              DENALI_CTL_172_DATA 0x00010001
#define              DENALI_CTL_173_DATA 0x00000000
#define              DENALI_CTL_174_DATA 0x00000000
#define              DENALI_CTL_175_DATA 0x00000000
#define              DENALI_CTL_176_DATA 0x00000000
#define              DENALI_CTL_177_DATA 0x00000000
#define              DENALI_CTL_178_DATA 0x00000000
#define              DENALI_CTL_179_DATA 0x00000000
#define              DENALI_CTL_180_DATA 0x00000000
#define              DENALI_CTL_181_DATA 0x01000000
#define              DENALI_CTL_182_DATA 0x00000001
#define              DENALI_CTL_183_DATA 0x00000100
#define              DENALI_CTL_184_DATA 0x00000101
#define              DENALI_CTL_185_DATA 0x67676701
#define              DENALI_CTL_186_DATA 0x67676767
#define              DENALI_CTL_187_DATA 0x67676767
#define              DENALI_CTL_188_DATA 0x67676767
#define              DENALI_CTL_189_DATA 0x67676767
#define              DENALI_CTL_190_DATA 0x67676767
#define              DENALI_CTL_191_DATA 0x67676767
#define              DENALI_CTL_192_DATA 0x67676767
#define              DENALI_CTL_193_DATA 0x67676767
#define              DENALI_CTL_194_DATA 0x01000067
#define              DENALI_CTL_195_DATA 0x00000001
#define              DENALI_CTL_196_DATA 0x00000101
#define              DENALI_CTL_197_DATA 0x00000000
#define              DENALI_CTL_198_DATA 0x00000000
#define              DENALI_CTL_199_DATA 0x00000000
#define              DENALI_CTL_200_DATA 0x00000000
#define              DENALI_CTL_201_DATA 0x00000000
#define              DENALI_CTL_202_DATA 0x00000000
#define              DENALI_CTL_203_DATA 0x00000000
#define              DENALI_CTL_204_DATA 0x00000000
#define              DENALI_CTL_205_DATA 0x00000000
#define              DENALI_CTL_206_DATA 0x00000000
#define              DENALI_CTL_207_DATA 0x00000000
#define              DENALI_CTL_208_DATA 0x00000001
#define              DENALI_CTL_209_DATA 0x00000000
#define              DENALI_CTL_210_DATA 0x007fffff
#define              DENALI_CTL_211_DATA 0x00000000
#define              DENALI_CTL_212_DATA 0x007fffff
#define              DENALI_CTL_213_DATA 0x00000000
#define              DENALI_CTL_214_DATA 0x007fffff
#define              DENALI_CTL_215_DATA 0x00000000
#define              DENALI_CTL_216_DATA 0x007fffff
#define              DENALI_CTL_217_DATA 0x00000000
#define              DENALI_CTL_218_DATA 0x007fffff
#define              DENALI_CTL_219_DATA 0x00000000
#define              DENALI_CTL_220_DATA 0x007fffff
#define              DENALI_CTL_221_DATA 0x00000000
#define              DENALI_CTL_222_DATA 0x007fffff
#define              DENALI_CTL_223_DATA 0x00000000
#define              DENALI_CTL_224_DATA 0x037fffff
#define              DENALI_CTL_225_DATA 0xffffffff
#define              DENALI_CTL_226_DATA 0x000f000f
#define              DENALI_CTL_227_DATA 0x00ffff03
#define              DENALI_CTL_228_DATA 0x000fffff
#define              DENALI_CTL_229_DATA 0x0003000f
#define              DENALI_CTL_230_DATA 0xffffffff
#define              DENALI_CTL_231_DATA 0x000f000f
#define              DENALI_CTL_232_DATA 0x00ffff03
#define              DENALI_CTL_233_DATA 0x000fffff
#define              DENALI_CTL_234_DATA 0x0003000f
#define              DENALI_CTL_235_DATA 0xffffffff
#define              DENALI_CTL_236_DATA 0x000f000f
#define              DENALI_CTL_237_DATA 0x00ffff03
#define              DENALI_CTL_238_DATA 0x000fffff
#define              DENALI_CTL_239_DATA 0x0003000f
#define              DENALI_CTL_240_DATA 0xffffffff
#define              DENALI_CTL_241_DATA 0x000f000f
#define              DENALI_CTL_242_DATA 0x00ffff03
#define              DENALI_CTL_243_DATA 0x000fffff
#define              DENALI_CTL_244_DATA 0x6407000f
#define              DENALI_CTL_245_DATA 0x01640001
#define              DENALI_CTL_246_DATA 0x00000000
#define              DENALI_CTL_247_DATA 0x00000000
#define              DENALI_CTL_248_DATA 0x00001700
#define              DENALI_CTL_249_DATA 0x00386c05
#define              DENALI_CTL_250_DATA 0x02000200
#define              DENALI_CTL_251_DATA 0x02000200
#define              DENALI_CTL_252_DATA 0x0000386c
#define              DENALI_CTL_253_DATA 0x00023438
#define              DENALI_CTL_254_DATA 0x02020d10
#define              DENALI_CTL_255_DATA 0x00140303
#define              DENALI_CTL_256_DATA 0x00000000
#define              DENALI_CTL_257_DATA 0x00000000
#define              DENALI_CTL_258_DATA 0x00001403
#define              DENALI_CTL_259_DATA 0x00000000
#define              DENALI_CTL_260_DATA 0x00000000
#define              DENALI_CTL_261_DATA 0x00000000
#define              DENALI_CTL_262_DATA 0x00000000
#define              DENALI_CTL_263_DATA 0x0d010000
#define              DENALI_CTL_264_DATA 0x00000008
