/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef __SOC_QUALCOMM_QCS405_ADDRESS_MAP_H__
#define __SOC_QUALCOMM_QCS405_ADDRESS_MAP_H__

#define QSPI_BASE			0x88DF000
#define TLMM_EAST_TILE_BASE		0x7B00000
#define TLMM_NORTH_TILE_BASE		0x1300000
#define TLMM_SOUTH_TILE_BASE		0x1000000
#define GCC_BASE			0x01800000

/* SDHC TLMM Registers */
#define SDC1_TLMM_CFG_ADDR		0x010C2000

#endif /*  __SOC_QUALCOMM_QCS405_ADDRESS_MAP_H__ */
