/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef MAINBOARD_DISPLAY_CARD_TYPE_H
#define MAINBOARD_DISPLAY_CARD_TYPE_H

#include <types.h>

uint8_t get_ddi1_type(void);

#endif /* MAINBOARD_DISPLAY_CARD_TYPE_H */
