/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef VARIANT_GPIO_H
#define VARIANT_GPIO_H

#include <baseboard/gpio.h>

/* Memory configuration board straps */
#define GPIO_MEM_CONFIG_0	GPP_F20
#define GPIO_MEM_CONFIG_1	GPP_F21
#define GPIO_MEM_CONFIG_2	GPP_F11
#define GPIO_MEM_CONFIG_3	GPP_F22

#endif
