/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef EC_CLEVO_IT5570E_EARLY_INIT_H
#define EC_CLEVO_IT5570E_EARLY_INIT_H

void ec_configure_kbled_booteffect(void);

#endif /* EC_CLEVO_IT5570E_EARLY_INIT_H */
