/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef _SOC_APOLLOLAKE_ITSS_H_
#define _SOC_APOLLOLAKE_ITSS_H_

#include <intelblocks/itss.h>

#define GPIO_IRQ_START	50
#define GPIO_IRQ_END	ITSS_MAX_IRQ

#endif /* _SOC_APOLLOLAKE_ITSS_H_ */
