/* SPDX-License-Identifier: GPL-2.0-only OR MIT */

#ifndef __SOC_MEDIATEK_MT8189_INCLUDE_SOC_ADDRESSMAP_H__
#define __SOC_MEDIATEK_MT8189_INCLUDE_SOC_ADDRESSMAP_H__

enum {
	IO_PHYS = 0x10000000,
};

enum {
	CKSYS_BASE			= IO_PHYS + 0x00000000,
	INFRACFG_AO_BASE		= IO_PHYS + 0x00001000,
	GPIO_BASE			= IO_PHYS + 0x00005000,
	APMIXED_BASE			= IO_PHYS + 0x0000C000,
	DEVAPC_INFRA_SECU_AO_BASE	= IO_PHYS + 0x0001C000,
	BCRM_INFRA_AO_BASE		= IO_PHYS + 0x00022000,
	DEVAPC_INFRA_AO_BASE		= IO_PHYS + 0x00030000,
	DEVAPC_INFRA_AO1_BASE		= IO_PHYS + 0x00034000,
	EMI0_BASE			= IO_PHYS + 0x00219000,
	EMI0_MPU_BASE			= IO_PHYS + 0x00226000,
	DRAMC_CHA_AO_BASE		= IO_PHYS + 0x00230000,
	THERM_CTRL_BASE			= IO_PHYS + 0x00315000,
	DPM_PM_SRAM_BASE		= IO_PHYS + 0x00900000,
	DPM_DM_SRAM_BASE		= IO_PHYS + 0x00920000,
	DPM_CFG_BASE			= IO_PHYS + 0x00940000,
	DPM_PM_SRAM_BASE2		= IO_PHYS + 0x00A00000,
	DPM_DM_SRAM_BASE2		= IO_PHYS + 0x00A20000,
	DPM_CFG_BASE2			= IO_PHYS + 0x00A40000,
	UART0_BASE			= IO_PHYS + 0x01001000,
	SFLASH_REG_BASE			= IO_PHYS + 0x01018000,
	PERICFG_AO_BASE			= IO_PHYS + 0x01036000,
	DEVAPC_PERI_PAR_AO_BASE		= IO_PHYS + 0x0103C000,
	SSUSB_IPPC_BASE			= IO_PHYS + 0x01203E00,
	UFSHCI_BASE			= IO_PHYS + 0x012B0000,
	MIPITX0_BASE			= IO_PHYS + 0x01B40000,
	IOCFG_LM_BASE			= IO_PHYS + 0x01B50000,
	EDP_BASE			= IO_PHYS + 0x01B70000,
	IOCFG_RB0_BASE			= IO_PHYS + 0x01C50000,
	IOCFG_RB1_BASE			= IO_PHYS + 0x01C60000,
	IOCFG_BM0_BASE			= IO_PHYS + 0x01D20000,
	IOCFG_BM1_BASE			= IO_PHYS + 0x01D30000,
	IOCFG_BM2_BASE			= IO_PHYS + 0x01D40000,
	IOCFG_LT0_BASE			= IO_PHYS + 0x01E20000,
	IOCFG_LT1_BASE			= IO_PHYS + 0x01E30000,
	SSUSB_SIF_BASE			= IO_PHYS + 0x01E80300,
	IOCFG_RT_BASE			= IO_PHYS + 0x01F20000,
	DSI0_BASE			= IO_PHYS + 0x04016000,
	DISP_DVO0			= IO_PHYS + 0x04019000,
	RGU_BASE			= IO_PHYS + 0x0C00A000,
	SPMI_MST_BASE			= IO_PHYS + 0x0C013000,
	DEVAPC_VLP_AO_BASE		= IO_PHYS + 0x0C018000,
	SPMI_MST_P_BASE			= IO_PHYS + 0x0CC00000,
	PMIF_SPMI_BASE			= IO_PHYS + 0x0CC04000,
	PMIF_SPMI_P_BASE		= IO_PHYS + 0x0CC06000,
	SYSTIMER_BASE			= IO_PHYS + 0x0CC10000,
	DEVAPC_MM_AO_BASE		= IO_PHYS + 0x0E820000,
};

#endif /* __SOC_MEDIATEK_MT8189_INCLUDE_SOC_ADDRESSMAP_H__ */
