/* SPDX-License-Identifier: GPL-2.0-only */

#ifndef AMD_BLOCK_POST_CODES_H
#define AMD_BLOCK_POST_CODES_H

#define POSTCODE_AGESA_AMDINITRESET			0x37
#define POSTCODE_AGESA_AMDINITEARLY			0x38

#define POSTCODE_ROMSTAGE_MAIN				0x40

#define POSTCODE_AGESA_AMDINITPOST			0x40
#define POSTCODE_AGESA_AMDINITPOST_DONE			0x41

#define POSTCODE_PSP_NOTIFY_DRAM			0x42
#define POSTCODE_PSP_NOTIFY_DRAM_DONE			0x43

#define POSTCODE_ROMSTAGE_RUN_POSTCAR			0x44

#define POSTCODE_PSP_LOAD_SMU				0x46
#define POSTCODE_AGESA_AMDINITENV			0x47
#define POSTCODE_AGESA_AMDS3LATERESTORE			0x48

#define POSTCODE_AGESA_AMDINITRESUME			0x60
#define POSTCODE_AGESA_AMDINITRESUME_DONE		0x61

#define POSTCODE_BOOTBLOCK_SOC_EARLY_INIT		0x90


#define POSTCODE_BOOTBLOCK_RESUME_ENTRY			0xb0
#define POSTCODE_BOOTBLOCK_PRE_C_ENTRY			0xa0
#define POSTCODE_BOOTBLOCK_PRE_C_DONE			0xa2

#endif
