/* SPDX-License-Identifier: CC-PDDC */

/* Please update this license if any licensable material is added. */

/* Generated automatically by the program `build/gcov-iov'
   from `4.7.2 (4 7) and  (*)'.  */

#define GCOV_VERSION ((gcov_unsigned_t)0x3430372a)  /* 407* */
