-- SPDX-License-Identifier: GPL-2.0-or-later

with HW.GFX.GMA;
with HW.GFX.GMA.Display_Probing;

use HW.GFX.GMA;
use HW.GFX.GMA.Display_Probing;

private package GMA.Mainboard is

   ports : constant Port_List :=
     (DP1,  -- DisplayPort connector
      HDMI1, -- DisplayPort connector, needed for passive DisplayPort to DVI/HDMI adapters
      DP2,  -- HDMI connector through Parade DP175 LSPCON
      others => Disabled);

end GMA.Mainboard;
